package ecdsa

import (
	"crypto"
	"crypto/internal/randutil"
	"io"

	boring "crypto/internal/backend"
)

func HashSign(rand io.Reader, priv *PrivateKey, msg []byte, h crypto.Hash) ([]byte, error) {
	randutil.MaybeReadByte(rand)

	if boring.Enabled() {
		b, err := boringPrivateKey(priv)
		if err != nil {
			return nil, err
		}
		return boring.HashSignECDSA(b, h, msg)
	}
	boring.UnreachableExceptTests()

	hash := h.New()
	hash.Write(msg)
	d := hash.Sum(nil)

	return SignASN1(rand, priv, d)
}

func HashVerify(pub *PublicKey, msg, sig []byte, h crypto.Hash) bool {
	if boring.Enabled() {
		bpk, err := boringPublicKey(pub)
		if err != nil {
			return false
		}
		return boring.HashVerifyECDSA(bpk, h, msg, sig)
	}
	boring.UnreachableExceptTests()

	hash := h.New()
	hash.Write(msg)
	d := hash.Sum(nil)

	return VerifyASN1(pub, d, sig)
}
